@@ -4,19 +4,25 @@
 setMethod("read.dataset",
           c("BNDataset", "character", "character"),
           function(object, data.file, header.file, data.with.header = FALSE,
-                   na.string.symbol = '?', sep.symbol = '', starts.from = 1,
+                   na.string.symbol = '?', sep.symbol = "", starts.from = 1,
                    num.time.steps = 1)
           {
             header.file(object)  <- header.file
             data.file(object)    <- data.file
 
-            a <- read.delim(data.file, na.strings = na.string.symbol,
+            a <- read.table(data.file, na.strings = na.string.symbol,
                             header = data.with.header, sep = sep.symbol) + (1 - starts.from)
             a <- as.matrix(a)
             
             ls                   <- readLines(header.file)
 
-            vars <- gsub('"', '', c(unlist(strsplit(ls[1], split = " "))))
+            #vars <- gsub('"', '', c(unlist(strsplit(ls[1], split = " "))))
+            # modified to keep spaces in variable names
+            # hat tip to https://stat.ethz.ch/pipermail/r-help/2012-November/342314.html
+            res<-unlist(strsplit(ls[1],"[\"]"))
+            res1<-res[res!=" "]
+            res1<-res1[res1!=""]
+            vars<-c(unlist(strsplit(res1[grepl("\\s+$",res1)]," ")),res1[!grepl("\\s+$",res1)])
             if (length(vars) == ncol(a)) {
               variables(object) <- vars
             } else if (num.time.steps > 1 && length(vars) * num.time.steps == ncol(a)) {
@@ -28,25 +34,33 @@ setMethod("read.dataset",
               }
               variables(object) <- copyvars
             } else {
-              stop("Incoherent number of variables in the dataset header.")
+              stop("Incoherent number of variable names in the dataset header (mismatch between header and data).")
             }
             
-            lns                  <- c(unlist(strsplit(ls[2], split = " ")))
+            lns                  <- c(unlist(strsplit(ls[2], split = "\\s+")))
             if (length(lns) == ncol(a)) {
               node.sizes(object)   <- sapply(1:length(lns), FUN=function(x){ as.numeric(lns[x]) })
             } else if (num.time.steps > 1 && length(lns) * num.time.steps == ncol(a)) {
               node.sizes(object)   <- rep(sapply(1:length(lns), FUN=function(x){ as.numeric(lns[x]) }), num.time.steps)
             } else {
-              stop("Incoherent number of variables in the dataset header.")
+              stop("Incoherent number of variable cardinalities in the dataset header (mismatch between header and data).")
             }
             
-            disc <- c(unlist(strsplit(ls[3], split = " ")))
+            disc <- c(unlist(strsplit(ls[3], split = "\\s+")))
+            for (d in 1:length(disc)) {
+              if (disc[d] %in% c("d","D","T","TRUE")) disc[d] <- 'D'
+              else if (disc[d] %in% c("c","C","F","FALSE")) disc[d] <- 'C'
+              else {
+                bnstruct.log("Unrecognized status for variable ",variables(object)[d],", converting it to discrete.")
+                disc[d] <- 'D'
+              }
+            }
             if (length(disc) == ncol(a)) {
               discreteness(object) <- disc
             } else if (num.time.steps > 1 && length(disc) * num.time.steps == ncol(a)) {
               discreteness(object) <-rep(disc, num.time.steps)
             } else {
-              stop("Incoherent number of variables in the dataset header.")
+              stop("Incoherent number of variable status in the dataset header (mismatch between header and data).")
             }
             
             
